package com.android.td5;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

public class Sieve extends Activity {
	// Nested class for continuations passed as messages to the graphical thread
	private class TextViewMessage implements Runnable {
		// TextView to display text into
		private TextView tv;

		// String to update the text of the TextView
		private String s;

		TextViewMessage(TextView tv, String s) {
			this.tv = tv;
			this.s = s;
		}

		@Override
		public void run() {
			// Update the TextView's text
			tv.setText(s);
		}
	}

	// Marking delay menu items
    private static final int MENU_FAST = 0;
    private static final int MENU_SLOW = 1;
    private static final int MENU_QUIT = 4;

    // Handler to update graphical elements
 	private final Handler callbackHandler = new Handler();
 
 	// Widgets
	private TextView statusTextView;
	private TextView resultTextView;
	private EditText numberEditText;

	// Data and computation model of the application
	private SieveModel model;

	// Post a message to the graphical thread to set the text of a TextView
	private void print(TextView tv, String s) {
		callbackHandler.post(new TextViewMessage(tv, s));
	}

	// Set the text of the status TextView
	public void printStatus(String s) {
		print(statusTextView, s);
	}
	
	// Set the text of the result TextView
	public void printResult(String s) {
		print(resultTextView, s);
	}

	// Parse the number in the number Edit Text
	public int parseNumber() {
		int n = 0;
		try {
			// Grab the value inside the EditText and check this is an integer value
			n = Integer.parseInt(numberEditText.getText().toString());			
			// Print the work in progress message
			printStatus("Sieve of n = " + n + " in progress");
		} catch (NumberFormatException e) {
			// Update the TextViewResult
			printResult("Please enter an integer value");
		}

		if (n < 0)
			n = 0;
		if (n > SieveModel.MAX_NUMBER)
			n = SieveModel.MAX_NUMBER;

		return n;
	}

	// Create an anonymous implementation of OnClickListener
	private OnClickListener startListener = new OnClickListener() {
	    public void onClick(View v) {
	    	model.sieve();
		}
	};

	/* Question 6 only (optional)
	// Create an anonymous implementation of OnCheckedChangeListerner
	private OnCheckedChangeListener numThreadsListener = new OnCheckedChangeListener() {
		@Override
		public void onCheckedChanged(RadioGroup group, int checkedId) {
            // To be completed
		}
	};
	*/

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        numberEditText = (EditText)findViewById(R.id.numberEditText);
        statusTextView = (TextView)findViewById(R.id.statusTextView);
        resultTextView = (TextView)findViewById(R.id.resultTextView);

        model = new SieveModel(this);
        model.init();

        // Capture the Start button
        Button startButton = (Button)findViewById(R.id.startButton);
        // Register our onClick listener
        startButton.setOnClickListener(startListener);

        /* Question 6 only
        SieveSurfaceView sieveSurfaceView = (SieveSurfaceView)findViewById(R.id.sieveSurfaceView);
        sieveSurfaceView.setSieveModel(model);
        */
        
        /* Question 6 only (optional)
        RadioGroup numThreadsRadioGroup = (RadioGroup)findViewById(R.id.numThreadsRadioGroup);
        numThreadsRadioGroup.setOnCheckedChangeListener(numThreadsListener);
        */

        // Check whether we are recreating a previously destroyed instance
        if (savedInstanceState != null) {
        	// To be completed
        }
        
        Log.w(this.getClass().getName(), "onCreate");
}

    /**
     * Invoked when the Activity loses user focus.
     */
    @Override
    protected void onPause() {
        super.onPause();
    }

    /**
     * Notification that something is about to happen, to give the Activity a
     * chance to save state.
     *
     * @param outState a Bundle into which this Activity should save its state
     */
    @Override
    protected void onSaveInstanceState(Bundle outState) {
        // Have the View's thread save its state into our Bundle
        super.onSaveInstanceState(outState);
    }

    /**
     * Invoked during init to give the Activity a chance to set up its Menu.
     *
     * @param menu the Menu to which entries may be added
     * @return true
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);

        menu.add(0, MENU_FAST, 0, R.string.menu_fast);
        menu.add(0, MENU_SLOW, 0, R.string.menu_slow);
        menu.add(0, MENU_QUIT, 0, R.string.menu_quit);

        return true;
    }

    /**
     * Invoked when the user selects an item from the Menu.
     *
     * @param item the Menu entry which was selected
     * @return true if the Menu item was legit (and we consumed it), false
     *         otherwise
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
    	int delay = 0;
    	
    	switch (item.getItemId()) {
            case MENU_FAST:
                delay = 0;
            	model.setDelay(delay);
                printStatus("Set mark delay to " + delay + "ms");
                break;
            case MENU_SLOW:
                delay = 100; // 100ms
            	model.setDelay(delay);
                printStatus("Set mark delay to " + delay + "ms");
                break;
            case MENU_QUIT:
            	finish();
            	break;
            default:
            	return false;
        }
        
        return true;
    }
}
